from mailmate import TemplatedEmailMessage
from mailmate.exceptions import MissingBody
import pytest


def test_body_required():
    """
    You must define either a body or a template.
    """
    with pytest.raises(MissingBody):
        TemplatedEmailMessage().message()


def test_empty_body():
    """
    If you explicitly set the body to an empty string, a MissingBody error
    shouldn't be raised.
    """
    TemplatedEmailMessage(body='').message()


def test_body_template():
    """
    The body string can be a template.
    """
    email = TemplatedEmailMessage(body='hello {{ name }}',
                                  extra_context={'name': 'world'})
    assert email.body == 'hello world'


def test_subject_template():
    """
    The subject can be a template.
    """
    email = TemplatedEmailMessage(subject='hello {{ name }}',
                                  body='',
                                  extra_context={'name': 'world'})
    assert email.message()['Subject'] == 'hello world'


def test_template():
    class Email(TemplatedEmailMessage):
        template_name = 'body.txt'

    assert Email(extra_context={'name': 'world'}).body.strip() == 'hello world'


def test_html_template():
    class Email(TemplatedEmailMessage):
        template_name = 'body.txt'
        html_template_name = 'body.html'

    email = Email(extra_context={'name': 'world'})
    assert '<p>hello world</p>' in email.message().as_string()


def test_autoplaintext():
    email = TemplatedEmailMessage(html_template_name='simple_body.html')
    assert email.body.strip() == 'hello world'



pkgmeta = dict(
    __title__='django-mailmate',
    __author__=
    'Chris McKenzie, Matthew Tretter, Colin Wood, Mohammad Taleb, Eric Eldredge',
    __version__='1.3.0', )

globals().update(pkgmeta)
__all__ = tuple(pkgmeta.keys())



from django.core.mail import EmailMultiAlternatives
from django.template import Template, Context, loader
from .exceptions import MissingBody

ALLOWED_TAGS = [
    'a', 'b', 'i', 'strong', 'em', 'hr', 'blockquote', 'br', 'ol', 'ul', 'li',
    'p', 'img'
] + ['h%s' % n for n in range(1, 7)]


class TemplatedEmailMessage(EmailMultiAlternatives):
    """
    example:
    In Emails.py

        from mailmate import TemplateMessage

        class CoolMessage(TemplateMessage):
            to = ['some-user@some-email.com']
            from_email = 'no-reply@some-email.com'
            subject = 'Super Cool Message'
            template_name = 'emails/text-template.txt'
            html_template_name = 'emails/html-template.html'

    In Views.py

        from .emails import CoolMessage

        message = CoolMessage(extra_context={'user': 'You', 'is': 'Cool'})
        message.send()
    """

    def __init__(self,
                 subject='',
                 body=None,
                 from_email=None,
                 to=None,
                 bcc=None,
                 connection=None,
                 attachments=None,
                 headers=None,
                 alternatives=None,
                 cc=None,
                 template_name=None,
                 html_template_name=None,
                 extra_context=None):

        self.subject_template = self._get_value('subject', subject)
        self.body_template = self._get_value('body', body)
        from_email = self._get_value('from_email', from_email)
        to = self._get_value('to', to)
        bcc = self._get_value('bcc', bcc)
        connection = self._get_value('connection', connection)
        attachments = self._get_value('attachments', attachments)
        headers = self._get_value('headers', headers)
        alternatives = self._get_value('alternatives', alternatives)
        cc = self._get_value('cc', cc)
        kwargs = {}
        if cc:
            kwargs['cc'] = cc

        self.template_name = self._get_value('template_name', template_name)
        self.html_template_name = self._get_value('html_template_name',
                                                  html_template_name)
        self.extra_context = self._get_value('extra_context', extra_context)

        alternatives = alternatives or []
        html_body = self.render_html_body()
        if html_body is not None:
            alternatives.append((html_body, 'text/html'))

        super(TemplatedEmailMessage, self).__init__(
            subject=self.render_subject(),
            body=self.render_body(),
            from_email=from_email,
            to=to,
            bcc=bcc,
            connection=connection,
            attachments=attachments,
            headers=headers,
            alternatives=alternatives,
            **kwargs)

    def get_context_data(self):
        return self.extra_context.copy() if self.extra_context else {}

    def get_context(self):
        return Context(self.get_context_data())

    def render_body(self):
        """
        Renders standard template with context
        """
        if self.body_template is not None:
            body = Template(self.body_template).render(self.get_context())
        elif self.template_name is not None:
            body = loader.get_template(self.template_name).render(
                self.get_context_data())
        else:
            try:
                body = self.body
            except AttributeError:
                body = None
            try:
                from markdownify import markdownify
            except ImportError:
                pass
            else:
                html_body = self.render_html_body()
                if html_body is not None:
                    body = markdownify(html_body, convert=ALLOWED_TAGS)
            if body is None:
                raise MissingBody('The email does not have a body. Either'
                                  ' provide a body or template_name or, if you'
                                  ' really want to send an email without a'
                                  ' body, set the body to an empty string'
                                  ' explicitly.')
        return body

    def render_subject(self):
        return Template(self.subject_template).render(self.get_context())

    def render_html_body(self):
        """
        Renders alternative template with context
        """
        if self.html_template_name:
            template = loader.get_template(self.html_template_name)
            return template.render(self.get_context_data())

    def _get_value(self, attr, value):
        return value or getattr(self.__class__, attr, None) or value



from django.core.mail.backends.filebased import EmailBackend
import email
import mimetypes
import os


class CleanEmailBackend(EmailBackend):
    """ Takes email messages and splits there parts into clean messages """
    '''
            Example
            --------

            .. code-block:: python

            #settings.py
            EMAIL_BACKEND = 'mailmate.backends.CleanEmailBackend'
            EMAIL_FILE_PATH = os.path.join(PROJECT_DIR, '../tmp', 'app-messages')

            Any email that is sent will be put in the project directorys tmp dir
            under the apps-messages directory. It will have three files for
            every email.

            A log with the whole email in it, a html file if there is a html
            template and a txt file for a txt template.
    '''

    def _write_file(self, contents, ext):
        filename, old_ext = os.path.splitext(self._get_filename())
        output_filename = u'%s%s' % (filename, ext)
        print(output_filename)
        fp = open(output_filename, 'wb')
        fp.write(contents)
        fp.close()

    def _write_message(self, message, content_type='text/plain'):
        if message.is_multipart():
            for m in message.get_payload():
                self._write_message(m)
        else:
            content_type = message.get_content_type()
            ext = '.txt' if content_type == 'text/plain' else mimetypes.guess_extension(
                content_type)
            if ext:
                self._write_file(message.get_payload(decode=True), ext)

    def close(self):
        if self.stream is not None:
            message = email.message_from_file(open(self._get_filename(), 'r'))
            self._write_message(message)
        super(CleanEmailBackend, self).close()



import os
from setuptools import setup, find_packages, Command
from setuptools.command.test import test as TestCommand
import sys

pkgmeta = {}
exec (open(
    os.path.join(os.path.dirname(__file__), 'mailmate', 'pkgmeta.py')).read(),
      pkgmeta)


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = ['tests', '-s']
        self.test_suite = True

    def run_tests(self):
        import pytest
        # Make sure this package's tests module gets priority.
        sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))
        os.environ['DJANGO_SETTINGS_MODULE'] = 'tests.settings'
        errno = pytest.main(self.test_args)
        sys.exit(errno)


class LintCommand(Command):
    """
    A copy of flake8's Flake8Command

    """
    description = "Run flake8 on modules registered in setuptools"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def distribution_files(self):
        if self.distribution.packages:
            for package in self.distribution.packages:
                yield package.replace(".", os.path.sep)

        if self.distribution.py_modules:
            for filename in self.distribution.py_modules:
                yield "%s.py" % filename

    def run(self):
        from flake8.engine import get_style_guide
        flake8_style = get_style_guide(config_file='setup.cfg')
        paths = self.distribution_files()
        report = flake8_style.check_files(paths)
        raise SystemExit(report.total_errors > 0)


setup(
    name='django-mailmate',
    version=pkgmeta['__version__'],
    author='Chris McKenzie',
    author_email='chrismc@hzdg.com',
    packages=find_packages(),
    include_package_data=False,
    description='Django email tools.',
    license='MIT',
    url='http://github.com/hzdg/django-mailmate',
    long_description=open('README.rst').read(),
    zip_safe=False,
    setup_requires=[],
    tests_require=[
        'pytest-django==2.6',
        'markdownify',
        'flake8',
    ],
    install_requires=[
        'Django>=1.8',
    ],
    extras_require={
        'autoplaintext': [
            'markdownify',
        ],
    },
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ],
    cmdclass={
        'test': PyTest,
        'lint': LintCommand,
    }, )
